import { Component, PureComponent, createElement } from 'react';
import { AutoSizer, Grid } from 'react-virtualized';
import { clamp, debounce, defaults, noop, throttle } from 'lodash';
import { Motion, spring } from 'react-motion';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = Object.setPrototypeOf ||
    ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
    function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

var wrap = function (index, length) { return (length + (index % length)) % length; };
var clamp$1 = function (index, min, max) { return Math.max(min, Math.min(index, max)); };

var SCROLL_THROTTLE = 50;
var SCROLL_END_TIMEOUT = 250;
var gridStyle = {
    outline: "none",
};
var SliderInner = /** @class */ (function (_super) {
    __extends(SliderInner, _super);
    function SliderInner(props) {
        var _this = _super.call(this) || this;
        _this.scrollDelta = 0;
        _this.totalScrollDelta = 0;
        _this.lastTouch = null;
        _this.bindRef = function (grid) {
            _this.el = grid;
        };
        _this.onMouseWheel = function (e) {
            if (!_this.props.scrollable)
                return;
            var xDist = Math.abs(e.deltaX);
            var yDist = Math.abs(e.deltaY);
            if (xDist / 2 < yDist)
                return; // ignore vertical scrolls
            var containerWidth = _this.el._scrollingContainer.clientWidth;
            var maxScrollLeft = _this.props.columnCount * _this.props.columnWidth - containerWidth;
            var nextScrollLeft = _this.scrollLeft + e.deltaX;
            if (nextScrollLeft > 0 && nextScrollLeft <= maxScrollLeft) {
                e.preventDefault(); // prevent "scroll bounce" when we're within range
            }
            _this.onSwipe(e.deltaX);
        };
        _this.onSwipe = function (deltaX) {
            _this.scrollDelta += deltaX;
            _this.totalScrollDelta += deltaX;
            _this.onScroll();
            _this.waitForScrollStop();
        };
        _this.onScroll = throttle(function () {
            var containerWidth = _this.el._scrollingContainer.clientWidth;
            var maxScrollLeft = _this.props.columnCount * _this.props.columnWidth - containerWidth;
            var nextScrollLeft = clamp(_this.state.scrollLeft + _this.scrollDelta, 0, maxScrollLeft);
            // check if this the start of a new scroll/slider was in sync
            if (_this.state.scrollLeft === _this.props.scrollLeft) {
                _this.props.onScrollStart();
            }
            _this.setState({ scrollLeft: nextScrollLeft });
            _this.scrollDelta = 0;
        }, SCROLL_THROTTLE);
        _this.waitForScrollStop = debounce(function () {
            var nextIndex = Math.round(_this.scrollLeft / _this.props.columnWidth);
            _this.lastTouch = null;
            _this.totalScrollDelta = 0;
            _this.props.onScroll(nextIndex);
        }, SCROLL_END_TIMEOUT);
        _this.onKeyDown = function (e) {
            var arrowKeys = ["ArrowUp", "ArrowRight", "ArrowDown", "ArrowLeft"];
            if (arrowKeys.includes(e.key)) {
                e.preventDefault();
                _this.props.onArrow(e);
            }
        };
        _this.getTouch = function (e) { return "touches" in e
            ? { x: e.touches[0].clientX, y: e.touches[0].clientY }
            : { x: e.clientX, y: e.clientY }; };
        _this.onTouchMove = function (e) {
            var _a = _this.props, swipeable = _a.swipeable, swipeForce = _a.swipeForce;
            if (!swipeable)
                return;
            var touch = _this.getTouch(e);
            if (_this.lastTouch !== null) {
                var deltaX = touch.x - _this.lastTouch.x;
                var deltaY = touch.y - _this.lastTouch.y;
                var xDist = Math.abs(deltaX);
                var yDist = Math.abs(deltaY);
                if (xDist / 2 < yDist) {
                    // ignore vertical swipes
                }
                else {
                    e.preventDefault();
                    _this.onSwipe(-deltaX * swipeForce);
                }
            }
            _this.lastTouch = touch;
        };
        _this.handleRest = function () {
            _this.props.onRest();
        };
        _this.recompute = function () {
            if (!_this.el)
                return;
            _this.el.recomputeGridSize();
        };
        _this.addKeydownListener = function () {
            /* workaround to ensure we only have one document listener at a time */
            /* necessary because edge cases can bypass the mouseLeave event */
            _this.removeKeydownListener();
            SliderInner.keyDownListener = _this.onKeyDown;
            document.addEventListener("keydown", SliderInner.keyDownListener);
        };
        _this.removeKeydownListener = function () {
            document.removeEventListener("keydown", SliderInner.keyDownListener);
        };
        _this.state = {
            scrollLeft: props.scrollLeft,
        };
        return _this;
    }
    Object.defineProperty(SliderInner.prototype, "scrollLeft", {
        get: function () {
            if (!this.el)
                return 0;
            return this.el._scrollingContainer.scrollLeft;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SliderInner.prototype, "columnWidth", {
        get: function () {
            return this.props.width / this.props.slidesToShow;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SliderInner.prototype, "synced", {
        get: function () {
            return this.state.scrollLeft === this.props.scrollLeft;
        },
        enumerable: true,
        configurable: true
    });
    SliderInner.prototype.componentWillReceiveProps = function (nextProps) {
        var isExternal = nextProps !== this.props;
        if (isExternal) {
            this.waitForScrollStop.flush();
        }
        if (nextProps.scrollLeft !== this.state.scrollLeft) {
            this.setState({ scrollLeft: nextProps.scrollLeft });
        }
    };
    SliderInner.prototype.componentDidUpdate = function () {
        this.el._scrollingContainer.style.overflow = "hidden";
        if (this.synced) {
            this.recompute();
        }
    };
    SliderInner.prototype.render = function () {
        var _this = this;
        var stopX = this.state.scrollLeft;
        return (createElement("div", { onWheel: this.onMouseWheel, onTouchMove: this.onTouchMove, onMouseEnter: this.addKeydownListener, onMouseLeave: this.removeKeydownListener },
            createElement(Motion, { defaultStyle: { scrollLeft: this.scrollLeft }, style: {
                    scrollLeft: spring(stopX, { stiffness: 170, damping: 26 }),
                }, onRest: this.handleRest }, function (_a) {
                var scrollLeft = _a.scrollLeft;
                return createElement(Grid, { ref: _this.bindRef, columnCount: _this.props.columnCount, rowCount: 1, width: _this.props.width, height: _this.props.height, cellRenderer: _this.props.cellRenderer, rowHeight: _this.props.height, columnWidth: _this.columnWidth, scrollToAlignment: "start", scrollLeft: scrollLeft, scrollingResetTimeInterval: SCROLL_END_TIMEOUT, style: gridStyle });
            })));
    };
    return SliderInner;
}(Component));

var Slider = /** @class */ (function (_super) {
    __extends(Slider, _super);
    function Slider() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.el = null;
        _this.scrollDelta = 0;
        _this.totalScrollDelta = 0;
        _this.renderSlide = function (_a) {
            var _b = _a.columnIndex, index = _b === void 0 ? 0 : _b, _c = _a.key, key = _c === void 0 ? "" : _c, _d = _a.rowIndex, rowIndex = _d === void 0 ? 0 : _d, _e = _a.style, style = _e === void 0 ? {} : _e;
            var _f = _this.props, slides = _f.slides, renderSlide = _f.children;
            var slide = slides[index];
            return (createElement("div", { className: "slide", key: key, style: __assign({}, _this.props.styleContainer({ slide: slide, index: index }), style) }, renderSlide({ slide: slide, index: index })));
        };
        return _this;
    }
    Object.defineProperty(Slider.prototype, "slideWidth", {
        get: function () {
            var _a = this.props, width = _a.width, slidesToShow = _a.slidesToShow;
            return width / slidesToShow;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Slider.prototype, "maxScrollLeft", {
        get: function () {
            var _a = this.props, slides = _a.slides, slidesToShow = _a.slidesToShow;
            return (slides.length * this.slideWidth) - (slidesToShow * this.slideWidth);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Slider.prototype, "scrollLeft", {
        get: function () {
            var scrollLeft = this.props.index * this.slideWidth;
            return clamp$1(scrollLeft, 0, this.maxScrollLeft);
        },
        enumerable: true,
        configurable: true
    });
    Slider.prototype.render = function () {
        var _this = this;
        var _a = this.props, slides = _a.slides, width = _a.width, height = _a.height, slidesToShow = _a.slidesToShow, onScrollStart = _a.onScrollStart, onScroll = _a.onScroll, onRest = _a.onRest, onArrow = _a.onArrow, scrollable = _a.scrollable, swipeable = _a.swipeable, className = _a.className, swipeForce = _a.swipeForce;
        return (createElement("div", { className: className },
            createElement(AutoSizer, { disableHeight: true, onResize: this.props.onResize }, function () { return (createElement(SliderInner, { columnCount: slides.length, width: width, height: height, cellRenderer: _this.renderSlide, rowHeight: height, columnWidth: width / slidesToShow, scrollLeft: _this.scrollLeft, slidesToShow: slidesToShow, onScroll: onScroll, onRest: onRest, onScrollStart: onScrollStart, scrollable: scrollable, swipeable: swipeable, onArrow: onArrow, swipeForce: swipeForce })); })));
    };
    return Slider;
}(PureComponent));
Slider.defaultProps = {
    index: 0,
    columnCount: 0,
    width: 0,
    height: 0,
    rowHeight: 0,
    slidesToShow: 1,
    swipeForce: 3,
    style: {},
    scrollable: false,
    swipeable: false,
    children: function (obj) { return undefined; },
    styleContainer: function (obj) { },
    onScroll: function (index) { return undefined; },
    onScrollStart: function () { return undefined; },
    onRest: function () { return undefined; },
    onArrow: function (e) { return undefined; },
    onResize: function (dimensions) { return undefined; },
};

var defaultScrollState = {
    index: 0,
    numSlides: 1,
    slidesToShow: 1,
    wraps: true,
};
var Resolver = /** @class */ (function () {
    function Resolver(getScrollState, onUpdate) {
        this.update = onUpdate;
        this.getScrollState = getScrollState;
    }
    Object.defineProperty(Resolver.prototype, "firstRight", {
        /* Getters */
        get: function () {
            var slidesToShow = this.getScrollState().slidesToShow;
            return slidesToShow - 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Resolver.prototype, "lastLeft", {
        get: function () {
            var _a = this.getScrollState(), numSlides = _a.numSlides, slidesToShow = _a.slidesToShow;
            return numSlides - slidesToShow;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Resolver.prototype, "lastRight", {
        get: function () {
            var numSlides = this.getScrollState().numSlides;
            return numSlides - 1;
        },
        enumerable: true,
        configurable: true
    });
    /* Helpers */
    Resolver.prototype.isPartialEdge = function (index) {
        if (index < this.firstRight && index > 0)
            return true;
        return index > this.lastLeft && index < this.lastRight;
    };
    Resolver.prototype.isFirstPage = function (index) {
        return index <= this.firstRight;
    };
    Resolver.prototype.isLastPage = function (index) {
        return index >= this.lastLeft;
    };
    Resolver.prototype.lastSlideIsVisibleAt = function (index) {
        return index >= this.lastLeft && index <= this.lastRight;
    };
    Resolver.prototype.firstSlideIsVisibleAt = function (index) {
        return index >= 0 && index <= this.firstRight;
    };
    Resolver.prototype.wrapIndex = function (index) {
        var _a = this.getScrollState(), numSlides = _a.numSlides, wraps = _a.wraps;
        return wraps ? wrap(index, numSlides) : this.clampIndex(index);
    };
    Resolver.prototype.clampIndex = function (index) {
        return clamp$1(index, 0, this.lastRight);
    };
    Resolver.prototype.resolve = function (index) {
        return this.clampIndex(this.wrapIndex(index));
    };
    /* Sliders */
    Resolver.prototype.slideLeft = function (n) {
        if (n === void 0) { n = 1; }
        var index = this.getScrollState().index;
        var nextIndex = this.resolve(index - n);
        this.update(nextIndex);
        return nextIndex;
    };
    Resolver.prototype.slideRight = function (n) {
        if (n === void 0) { n = 1; }
        var index = this.getScrollState().index;
        var nextIndex = this.resolve(index + n);
        this.update(nextIndex);
        return nextIndex;
    };
    Resolver.prototype.pageLeft = function (n) {
        if (n === void 0) { n = 1; }
        var _a = this.getScrollState(), index = _a.index, slidesToShow = _a.slidesToShow, wraps = _a.wraps;
        var nextIndex = index - (n * slidesToShow);
        if (this.isFirstPage(index) && this.isLastPage(index)) {
            nextIndex = 0;
        }
        else if (this.isFirstPage(index)) {
            nextIndex = wraps ? this.lastLeft : 0;
        }
        this.update(nextIndex);
        return nextIndex;
    };
    Resolver.prototype.pageRight = function (n) {
        if (n === void 0) { n = 1; }
        var _a = this.getScrollState(), index = _a.index, slidesToShow = _a.slidesToShow, wraps = _a.wraps;
        var nextIndex = index + (n * slidesToShow);
        if (this.isFirstPage(index) && this.isLastPage(index)) {
            nextIndex = 0;
        }
        else if (this.isLastPage(index)) {
            nextIndex = wraps ? 0 : this.lastLeft;
        }
        this.update(nextIndex);
        return nextIndex;
    };
    return Resolver;
}());
var createScroller = function (_getState, onUpdate) {
    if (_getState === void 0) { _getState = noop; }
    if (onUpdate === void 0) { onUpdate = noop; }
    var getState = function () { return defaults({}, _getState(), defaultScrollState); };
    return new Resolver(getState, onUpdate);
};

export { Slider, wrap, clamp$1 as clamp, createScroller };
